/**
 * OW2 Util
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.war.impl.xml.merge;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.ow2.util.ee.metadata.common.api.ICommonClassMetadata;
import org.ow2.util.ee.metadata.common.api.ICommonFieldMetadata;
import org.ow2.util.ee.metadata.common.api.ICommonMethodMetadata;
import org.ow2.util.ee.metadata.common.api.interfaces.IEnvironmentRefAccessor;
import org.ow2.util.ee.metadata.common.api.struct.IJAnnotationResource;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxPersistenceUnit;
import org.ow2.util.ee.metadata.common.api.struct.IJaxwsWebServiceRef;
import org.ow2.util.ee.metadata.common.impl.xml.merge.MetadataMerge;
import org.ow2.util.ee.metadata.war.api.IWarClassMetadata;
import org.ow2.util.ee.metadata.war.api.IWarFieldMetadata;
import org.ow2.util.ee.metadata.war.api.IWarMethodMetadata;
import org.ow2.util.ee.metadata.war.impl.WarDeployableMetadata;
import org.ow2.util.ee.metadata.war.impl.xml.struct.WAR;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Merge the Web.xml Deployment descriptor with the annotations that have been parsed.
 * @author Florent Benoit
 */
public class WarMetadataMerge extends MetadataMerge {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(WarMetadataMerge.class);

    /**
     * The metadata where merge will be done.
     */
    private WarDeployableMetadata warDeployableMetadata;

    /**
     * Constructor.
     * @param warDeployableMetadata the metadata where merge will be done
     */
    public WarMetadataMerge(final WarDeployableMetadata warDeployableMetadata) {
        this.warDeployableMetadata = warDeployableMetadata;
    }

    /**
     * Merge current metadata with war.
     * @param war object representing the xml
     */
    public void merge(final WAR war) {
        applyJndiEnvironmentRefsGroup(war, warDeployableMetadata);

        // merge into classes
        for (IWarClassMetadata warClassmetadata : warDeployableMetadata.getWarClassMetadataCollection()) {
            // Merge persistence units
            List<IJavaxPersistenceUnit> persistenceUnitsDeployable = warDeployableMetadata.getJavaxPersistencePersistenceUnits();
            if (persistenceUnitsDeployable != null && persistenceUnitsDeployable.size() > 0) {
                List<IJavaxPersistenceUnit> persistenceUnitsClass = warClassmetadata.getJavaxPersistencePersistenceUnits();
                if (persistenceUnitsClass == null) {
                    warClassmetadata.setJavaxPersistencePersistenceUnits(persistenceUnitsDeployable);
                } else {
                    persistenceUnitsClass.addAll(persistenceUnitsDeployable);
                }
            }

            // Merge @Resources
            List<IJAnnotationResource> annotationResourcesDeployable = warDeployableMetadata.getJAnnotationResources();
            if (annotationResourcesDeployable != null && annotationResourcesDeployable.size() > 0) {
                List<IJAnnotationResource> annotationResourcesClass = warClassmetadata.getJAnnotationResources();
                if (annotationResourcesClass == null) {
                    warClassmetadata.setJAnnotationResources(annotationResourcesDeployable);
                } else {
                    annotationResourcesClass.addAll(annotationResourcesDeployable);
                }
            }
        }
    }

    /**
     * Needs to search the WebServiceRef in all fields/methods of the WAR.
     * @param warMetadata the war metadata
     * @param name the xml name of the service ref
     * @return a list of matching web service ref
     */
    @Override
    protected List<IJaxwsWebServiceRef> findJaxwsWebServiceRefMetadata(final IEnvironmentRefAccessor warMetadata,
            final String name) {

        List<IJaxwsWebServiceRef> elements = new ArrayList<IJaxwsWebServiceRef>();

        Collection<IWarClassMetadata> clientClassesMetadata = warDeployableMetadata.getWarClassMetadataCollection();
        if (clientClassesMetadata != null) {
            for (IWarClassMetadata classMetadata : clientClassesMetadata) {
                Collection<? extends IWarFieldMetadata> fieldMetadatas = classMetadata.getStandardFieldMetadataCollection();
                if (fieldMetadatas != null) {
                    for (IWarFieldMetadata fieldMetadata : fieldMetadatas) {
                        addJaxwsWebServiceRefIfFound(fieldMetadata, name, elements);
                    }
                }
                Collection<? extends IWarMethodMetadata> methodMetadatas = classMetadata.getStandardMethodMetadataCollection();
                if (methodMetadatas != null) {
                    for (IWarMethodMetadata methodMetadata : methodMetadatas) {
                        addJaxwsWebServiceRefIfFound(methodMetadata, name, elements);
                    }
                }
            }
        }
        return elements;
    }

    /**
     * @param className the class name
     * @return class metadata of class name
     */
    @Override
    public IWarClassMetadata getClassMetadata(final String className) {
        return warDeployableMetadata.getWarClassMetadata(className);
    }

    /**
     * No Main-Class for War archive.
     * @param env the car environment
     * @return null
     */
    @Override
    public String findDefaultClassForEnvironment(final IEnvironmentRefAccessor env) {
        return null;
    }

    /**
     * @return A collection of all ClassMetadata of the merged object.
     */
    protected Collection<? extends ICommonClassMetadata> getAllClassMetadatas() {
        return warDeployableMetadata.getWarClassMetadataCollection();
    }

    /**
     * @return A collection of all MethodMetadata of the merged object.
     */
    protected Collection<? extends ICommonMethodMetadata> getAllMethodMetadatas() {
        // Init the returned List
        Collection<ICommonMethodMetadata> methodMetadatas = new ArrayList<ICommonMethodMetadata>();

        Collection<? extends ICommonClassMetadata> classMetadatas = warDeployableMetadata.getWarClassMetadataCollection();
        for (ICommonClassMetadata classMetadata : classMetadatas) {
            methodMetadatas.addAll(classMetadata.getStandardMethodMetadataCollection());
        }
        return methodMetadatas;
    }

    /**
     * @return A collection of all FieldMetadata of the merged object.
     */
    protected Collection<? extends ICommonFieldMetadata> getAllFieldMetadatas() {
        // Init the returned List
        Collection<ICommonFieldMetadata> fieldMetadatas = new ArrayList<ICommonFieldMetadata>();

        Collection<? extends ICommonClassMetadata> classMetadatas = warDeployableMetadata.getWarClassMetadataCollection();
        for (ICommonClassMetadata classMetadata : classMetadatas) {
            fieldMetadatas.addAll(classMetadata.getStandardFieldMetadataCollection());
        }
        return fieldMetadatas;
    }

}
